package uy.edu.fing.tse.jsf;

import org.primefaces.PrimeFaces;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import uy.edu.fing.tse.central.business.BusinessLocal;
import uy.edu.fing.tse.central.business.security.SecurityLocal;
import uy.edu.fing.tse.dto.Role;
import uy.edu.fing.tse.dto.UserBO;

import javax.annotation.PostConstruct;
import javax.ejb.EJB;
import javax.enterprise.context.RequestScoped;
import javax.faces.application.FacesMessage;
import javax.faces.context.FacesContext;
import javax.inject.Named;
import javax.validation.constraints.NotEmpty;
import javax.validation.constraints.Size;
import java.io.Serializable;
import java.net.URLEncoder;
import java.nio.charset.StandardCharsets;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

@RequestScoped
@Named("userLoginView")
public class UserLoginBean implements Serializable {

    private static final long serialVersionUID = -7674319505640122631L;

    private static final Logger LOG = LoggerFactory.getLogger(UserLoginBean.class);
    @EJB
    private SecurityLocal securityLocal;

    @EJB
    private BusinessLocal negocio;

    private List<Role> roles;

    @PostConstruct
    public void init() {
        roles = negocio.listarRoles();
    }

    final UserBO user = new UserBO();

    @NotEmpty
    @Size(min = 8, message = "Password must have at least 8 characters")
    private String repassword;

    public List<Role> getRoles() {
        return roles;
    }

    public void setRoles(List<Role> roles) {
        this.roles = roles;
    }

    public UserBO getUser() {
        return user;
    }

    public String getRepassword() {
        return repassword;
    }

    public void setRepassword(String repassword) {
        this.repassword = repassword;
    }

    public void login() {
        String token = securityLocal.login(user);

        if (token != null) {
            addCookieToken(token);
            PrimeFaces.current().executeScript("window.location.href='jsf/listarHechos.xhtml'");
        } else {
            FacesContext.getCurrentInstance().addMessage(
                    null,
                    new FacesMessage(FacesMessage.SEVERITY_WARN,
                            "Ingreso Incorrecto",
                            "Por favor verificar los datos ingresados"));
        }
    }

    public void logout() {
        removeCookieToken();
        FacesContext.getCurrentInstance().getExternalContext().invalidateSession();
        PrimeFaces.current().executeScript("window.location.href='jsf/index.xhtml'");
    }

    public void register() {
        boolean valid = true;

        // verificar que sean iguales la pasw, en caso contrario
        valid &= user.getPassword().equals(repassword);
        //Verificar que no existe usuario
        //TODO
        if (valid) {
            final var s = securityLocal.register(user);
            PrimeFaces.current().executeScript("window.location.href='jsf/listarHechos.xhtml'");
        } else {
            FacesContext.getCurrentInstance().addMessage(
                    null,
                    new FacesMessage(FacesMessage.SEVERITY_WARN,
                            " Usuario o Password diferentes",
                            "Por favor verificar los datos ingresados"));
        }
    }

    private void addCookieToken(final String token) {
        final Map<String, Object> properties = new HashMap<>();
        properties.put("maxAge", 31536000);
        properties.put("path", "/");
        FacesContext.getCurrentInstance().getExternalContext()
                .addResponseCookie("token", URLEncoder.encode(token, StandardCharsets.UTF_8), properties);
    }

    private void removeCookieToken() {
        final Map<String, Object> properties = new HashMap<>();
        properties.put("maxAge", 0);
        properties.put("path", "/");
        FacesContext.getCurrentInstance().getExternalContext()
                .addResponseCookie("token", "", properties);
    }
}
