package uy.edu.fing.tse.central.db.dao.role;

import uy.edu.fing.tse.central.db.entity.Rol;
import uy.edu.fing.tse.central.db.mapper.MyMapper;
import uy.edu.fing.tse.dto.Role;

import javax.annotation.PostConstruct;
import javax.ejb.Stateless;
import javax.persistence.EntityManager;
import javax.persistence.PersistenceContext;
import java.util.List;
import java.util.stream.Collectors;

@Stateless
public class RoleDAOBean implements RoleDaoLocal {

    @PersistenceContext(unitName = "central")
    private EntityManager em;

    @PostConstruct
    void init() {
        //Despues del constructor hace esto
    }

    @Override
    public Role create(Role elem) {
        final var r = MyMapper.INSTANCE.convert(elem);
        em.persist(r);
        em.flush();
        return MyMapper.INSTANCE.convert(r);
    }

    @Override
    public Role update(Role elem) {
        final var r = MyMapper.INSTANCE.convert(elem);
        final var then = em.merge(r);
        em.flush();
        return MyMapper.INSTANCE.convert(then);
    }

    @Override
    public void remove(Role elem) {
        final var r = em.find(Rol.class, elem.getId());
        em.remove(r);
    }
    @Override
    public Role find(String key) {
        final var query = em.createNamedQuery("Rol.findByName", Rol.class);
        query.setParameter("name", key);
        final var r = query.getSingleResult();
        return MyMapper.INSTANCE.convert(r);
    }

    @Override
    public Role find(Long id) {
        final var r = em.find(Rol.class, id);
        return MyMapper.INSTANCE.convert(r);
    }

    @Override
    public List<Role> findAll() {
        final var query = em.createNamedQuery("Rol.findAll", Rol.class);
        final List<Rol> r = query.getResultList();
        return r.stream().map(MyMapper.INSTANCE::convert).collect(Collectors.toList());
    }
}
