package uy.edu.fing.tse.central.db.dao.user;

import uy.edu.fing.tse.central.db.entity.Usuario;
import uy.edu.fing.tse.central.db.mapper.MyMapper;
import uy.edu.fing.tse.dto.User;

import javax.annotation.PostConstruct;
import javax.ejb.Stateless;
import javax.persistence.EntityManager;
import javax.persistence.PersistenceContext;
import java.util.List;
import java.util.stream.Collectors;

@Stateless
public class UserDAOBean implements UserDaoLocal {

    @PersistenceContext(unitName = "central")
    private EntityManager em;

    @PostConstruct
    void init() {
        //Despues del constructor hace esto
    }

    @Override
    public User create(User user) {
        final var newUser = MyMapper.INSTANCE.convert(user);
        em.persist(newUser);
        em.flush();
        return MyMapper.INSTANCE.convert(newUser);
    }

    @Override
    public User update(User elem) {
        final var r = MyMapper.INSTANCE.convert(elem);
        final var then = em.merge(r);
        em.flush();
        return MyMapper.INSTANCE.convert(then);
    }

    @Override
    public void remove(User elem) {
        final var r = em.find(Usuario.class, elem.getId());
        em.remove(r);
    }

    @Override
    public User find(String mail) {
        final var query = em.createNamedQuery("Usuario.findByMail", Usuario.class);
        query.setParameter("mail", mail);
        final Usuario usuario = query.getSingleResult();
        return MyMapper.INSTANCE.convert(usuario);
    }

    @Override
    public User find(Long id) {
        final var e = em.find(Usuario.class, id);
        return MyMapper.INSTANCE.convert(e);
    }

    @Override
    public List<User> findAll() {
        final var query = em.createNamedQuery("Usuario.findAll", Usuario.class);
        final List<Usuario> usuarios = query.getResultList();

        return usuarios.stream().map(MyMapper.INSTANCE::convert).collect(Collectors.toList());
    }

}
