package uy.edu.fing.tse.central.business;


import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import uy.edu.fing.tse.central.db.dao.category.CategoryDaoLocal;
import uy.edu.fing.tse.central.db.dao.check.checker.CheckCheckerDaoLocal;
import uy.edu.fing.tse.central.db.dao.check.mechanism.CheckMechanismDaoLocal;
import uy.edu.fing.tse.central.db.dao.fact.FactDaoLocal;
import uy.edu.fing.tse.central.db.dao.mechanism.MechanismDaoLocal;
import uy.edu.fing.tse.central.db.dao.role.RoleDaoLocal;
import uy.edu.fing.tse.central.db.dao.score.ScoreDaoLocal;
import uy.edu.fing.tse.central.db.dao.state.StateDaoLocal;
import uy.edu.fing.tse.central.db.dao.statehistory.StateHistoryDaoLocal;
import uy.edu.fing.tse.central.db.dao.user.UserDaoLocal;
import uy.edu.fing.tse.dto.*;

import javax.ejb.EJB;
import javax.ejb.Stateless;
import java.util.List;

@Stateless
public class BusinessBean implements BusinessLocal, BusinessRemote {

    private static final long serialVersionUID = 4079626203846038482L;
    private static final Logger LOG = LoggerFactory.getLogger(Business.class);

    @EJB
    private CategoryDaoLocal categorias;
    @EJB
    private CheckCheckerDaoLocal verificacionesChecker;
    @EJB
    private CheckMechanismDaoLocal verificacionesMecanismos;
    @EJB
    private FactDaoLocal hechos;
    @EJB
    private MechanismDaoLocal mecanismos;
    @EJB
    private RoleDaoLocal roles;
    @EJB
    private ScoreDaoLocal calificaciones;
    @EJB
    private StateDaoLocal estados;
    @EJB
    private StateHistoryDaoLocal transiciones;
    @EJB
    private UserDaoLocal usuarios;


    @Override
    public void altaPeriferico(Peripherical p) {
        mecanismos.create(p);
    }

    @Override
    public void updateUsuario(User u) {
        usuarios.update(u);
    }

    @Override
    public void altaVerificacionMecanismo(CheckMechanism cm) {
        verificacionesMecanismos.create(cm);
    }

    @Override
    public Fact altaHecho(Fact fact) {

        final var state = estados.find(fact.getActualState().getValue());
        fact.setActualState(state);

        if (fact.getCategory() != null) {
            final var cat = categorias.find(fact.getCategory().getName());
            fact.setCategory(cat);
        }

        if (fact.getScore() != null) {
            final var score = calificaciones.find(fact.getScore().getValue());
            fact.setScore(score);
        }


        for (StateHistory sh : fact.getHistory()) {
            final var u = usuarios.find(sh.getUser().getMail());
            sh.setUser(u);

            final var s = estados.find(sh.getState().getValue());
            sh.setState(s);
        }

        return hechos.create(fact);
    }


    @Override
    public State altaEstado(State state) {
        return estados.create(state);
    }

    @Override
    public Role altaRol(Role role) {
        return roles.create(role);
    }

    @Override
    public Score altaCalificacion(Score e) {
        return calificaciones.create(e);
    }

    @Override
    public List<Fact> listarHechos() {
        return hechos.findAll();
    }

    @Override
    public List<User> listarUsuarios() {
        return usuarios.findAll();
    }

    @Override
    public List<Role> listarRoles() {
        return roles.findAll();
    }

    @Override
    public List<State> listarEstados() {
        return estados.findAll();
    }

    @Override
    public void altaUsuario(User user) {
        usuarios.create(user);
    }
}
